<?php // $Id: pay.inc,v 1.9 2009/08/21 03:23:33 vauxia Exp $

class pay {
  var $table = '';
  var $key = '';

  function __construct($values = NULL) {
    $current = (array) $this;
    if ($values) {
      foreach ((array)$values as $name => $val) {
        if ($name == 'settings' && !empty($val) && is_scalar($val)) {
          $this->__construct(unserialize($val));
        }
        elseif (array_key_exists($name, $current) && !is_null($val)) {
          $func = 'set_'. $name;
          if (method_exists($this, $func)) {
            $this->$func($val);
          }
          else {
            $this->$name = $val;
          }
          unset($current[$name]);
        }
      }
    }
    // Set defaults if possible.
    foreach ($current as $name => $val) {
      $func = 'set_'. $name;
      if (method_exists($this, $func)) {
        $this->$func($this->$name);
      }
    }
  }

  /**
   * Do not allow this value to be automatically set.
   */
  function set_key() {
    return;
  }

  /**
   * Do not allow this value to be automatically set.
   */
  function set_table() {
    return;
  }

  function set_activity($val = NULL) {
    $this->activity = check_plain($val);
  }

  function set_hostname($val = NULL) {
    if (!$val) $val = ip_address();
    $this->hostname = check_plain($val);
  }

  function set_notes($val = NULL) {
    $this->notes = filter_xss($val);
  }

  function set_pay_status($val = 'pending') {
    $this->pay_status = check_plain($val);
  }

  function set_status($status = 1) {
    $this->status = (int) (bool) $status;
  }

  function set_total($val = 0) {
    $this->total = round((float) $val, 2);
  }

  function set_total_paid($val = 0) {
    $this->total_paid = round((float) $val, 2);
  }

  function set_transaction_total($val = 0) {
    if (!$val) $val = $this->total;
    if ($this->result) {
      $this->transaction_total = (float) $val;
    }
  }

  function set_created($val = NULL) {
    if (!$val) $val = time();
    if (!is_int($val)) $val = strtotime($val);
    $this->created = (int)$val;
  }

  function set_timestamp($val = NULL) {
    if (!$val) $val = time();
    if (!is_int($val)) $val = strtotime($val);
    $this->timestamp = (int)$val;
  }

  function set_completed($val = NULL) {
    if ($val) {
      if (!is_int($val)) $val = strtotime($val);
      $this->completed = (int)$val;
    }
  }

  function set_uid($val = NULL) {
    if (!$val) {
      global $user;
      $val = $user->uid;
    }
    $this->uid = (int)$val;
  }

  function set_mail($val = NULL) {
    if (valid_email_address($val)) $this->mail = $val;
  }
  
  function set_handler() {
    $this->handler = get_class($this);
  }

  function handler() {
    $this->set_handler();
    return $this->handler;
  }
  
  function handler_title() {
    if ($info = pay_handlers($this->table, $this->handler())) {
      return filter_xss($info['title']);
    }
  }

  function menu_path() {
    if ($this->menu_path) return check_plain($this->menu_path);
  }
  
  function title() {
    return filter_xss($this->title);
  }
  
  function uid() {
    return (int) $this->uid;
  }

  function user() {
    return user_load(array('uid' => $this->uid()));
  }
  
  function total() {
    $total = (float) $this->total;
    if (strpos($total, '.') === FALSE) {
      $total .= '.00';
    }
    if (strpos($total, '.') === 0) {
      $total = '0.'. $total;
    }
    return $total;
  }

  function settings_form(&$form_state) {
    $form['handler'] = array(
      '#type' => 'value',
      '#value' => $this->handler(),
    );
    $key = $this->key;
    $form[$key] = array(
      '#type' => 'value',
      '#value' => $this->$key,
    );
    return $form;
  }

  function settings_form_validate($form, &$form_state) {
  }

  function settings_form_submit($form, &$form_state) {
    $this->save($form_state['values']);
  }
  
  function form(&$form_state) {
    return array();
  }

  function form_validate($form, &$form_state, $key) {
  }

  function form_submit($form, &$form_state, $key) {
  }

  function save($values = array()) {
    $this->__construct($values);
    $settings = (array) $this;
    $exclude = array('key', 'table');
    if (isset($this->table)) {
      $schema = drupal_get_schema($this->table);
      $settings = (array) $this;
      foreach ($this as $key => $val) {
        if (isset($schema['fields'][$key]) || in_array($key, $exclude)) {
          unset($settings[$key]);
        }
      }
      $this->settings = $settings;
      $this->handler = get_class($this);
      $key = $this->key;

      $update = $this->$key ? $key : NULL;
      drupal_write_record($this->table, $this, $update);
    }
  }
}
