<?php // $Id: pay_form.inc,v 1.7 2009/08/21 02:49:22 vauxia Exp $

/**
 * @file
 * The base class for payment activities.  All payment form classes should
 * extend this.
 */

class pay_form extends pay {

  var $pfid;
  var $title;
  var $status = 1;
  var $uid;
  var $min_amount = 1.00;
  var $max_amount = 1000.00;
  var $total_goal;
  var $currency;
  var $pay_methods = array();
  var $pay_form_activity = 'complete';
  var $notes_title = 'Comments';
  var $notes_format = FILTER_FORMAT_DEFAULT;
  var $notes_description = '';

  var $table = 'pay_form';
  var $key = 'pfid';

  function title() {
    return check_plain($this->title);
  }

  function set_transaction($values) {
    module_load_include('inc', 'pay', 'includes/handlers/pay_transaction');
    $this->transaction = New pay_transaction($values);
    if (!$this->transaction->pfid) $this->transaction->pfid = $this->pfid;
    $this->transaction->save();
    return $this->transaction;
  }

  function pay_method_list() {
    $list = array();
    $res = db_query("SELECT * FROM {pay_method} WHERE STATUS = 1 ORDER BY title");
    while ($row = db_fetch_object($res)) {
      $list[$row->pmid] = $row->title;
    }
    return $list;
  }

  function settings_form(&$form_state) {
    $form = parent::settings_form($form_state);
    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#default_value' => $this->title,
      '#required' => TRUE,
      '#description' => t('This title will appear as the title of the form page and in listings.')
    );
    
    $form['status'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enabled'),
      '#default_value' => $this->status,
    );

    $form['min_amount'] = array(
      '#type' => 'textfield',
      '#size' => 10,
      '#title' => t('Minimum amount'),
      '#description' => t('The minimum allowed payment on this form. If there are service fees or other costs associated with receiving payments, you will want this to set this amount higher than those costs.'),
      '#default_value' => $this->min_amount,
    );

    $form['max_amount'] = array(
      '#type' => 'textfield',
      '#size' => 10,
      '#title' => t('Maximum amount'),
      '#description' => t('The maximum allowed amount for this form.  If your payment method has a limit, this setting should reflect it.'),
      '#default_value' => $this->max_amount,
    );

    $form['pay_methods'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Available payment methods'),
      '#description' => t('Payment methods available for this form.'),
      '#options' => $this->pay_method_list(),
      '#default_value' => $this->pay_methods,
    );

    $form['pay_form_activity'] = array(
      '#type' => 'radios',
      '#title' => t('When this form is submitted'),
      '#options' => array(
        'complete' => t('Process payment immediately.'),
        'authorize' => t('Collect payment information and authorize the card (if applicable), but do not process payment.  Set the transaction to "Authorized".'),
        '' => t('Do not collect payment information or process payment, leave the transaction as "Pending".'),
      ),
      '#description' => t('In some cases, (for example, when delivering a product), you may be legally required to defer payment for review or further action.  If the payment method supports it, the transaction will be authorized but not processed.'),
      '#default_value' => $this->pay_form_activity,
    );
    
    return $form;
  }

  function pay_methods() {
    $methods = array();
    foreach ($this->pay_methods as $pmid => $status) {
      if ($status) $methods[$pmid] = pay_method_load($pmid);
    }
    return $methods;
  }

  function pay_method_form(&$form_state) {
    $form = array('#tree' => TRUE);
    $methods = $this->pay_methods();
    $options = array();
    foreach ($methods as $pmid => $pay_method) $options[$pmid] = $pay_method->title;

    $form['selected'] = array(
      '#type' => 'radios',
      '#options' => $options,
      '#required' => TRUE,
      '#access' => count($methods) > 1,
      '#default_value' => count($methods) == 1 ? $methods : NULL,
    );

    foreach ($methods as $pay_method) {
      $form[$pay_method->pmid] = $pay_method->form($form_state);
      $form[$pay_method->pmid]['pmid'] = array(
        '#type' => 'value', '#value' => $pay_method->pmid,
      );
    }
    return $form;
  }

  function form_validate($form, &$form_state, $key) {
    parent::form_validate($form, $form_state, $key);
    $values = $form_state['values'][$key];
    $total = (float) $values['total'];

    // Confirm that the amount falls within our min/max settings.
    if (($total < $this->min_amount) || ($total > $this->max_amount) ) {
      $error = t('Please enter an amount between %min and %max', array('%min' =>$this->min_amount, '%max' => $this->max_amount));
      form_set_error('total', $error);
    }

    // If there is no total value per payment method, set the global total.
    foreach ($values['pay_method']['selected'] as $pmid => $status) {
      if (!$status) continue;
      form_set_value($form[$key]['pay_method'][$pmid]['total'], $total, $form_state);
      $pay_method = pay_method_load($form_state['values'][$key]['pay_method'][$pmid]);
      $pay_method->pay_method_validate($form, $form_state, $key, $form[$key]['pay_method'][$pmid]);
    }
  }

  function form_submit($form, &$form_state, $key) {
    parent::form_submit($form, $form_state, $key);
    $values = $form_state['values'][$key];

    // Add a new transaction for this form with status as default of 'pending'.
    $transaction = $this->set_transaction($values);

    if ($activity_name = $this->pay_form_activity) {
      // Run payment activities for the 1 or more selected payment methods.
      $selected = $values['pay_method']['selected'];
      if (!is_array($selected)) $selected = array($selected => 1);

      foreach ($selected as $pmid => $status) {
        if (!$status) continue;
        $pay_method_values = $values['pay_method'][$pmid];
        $pay_method['pmid'] = $pmid;
        $pay_method = pay_method_load($pay_method_values);
        $transaction->save($pay_method_values);

        $activity = $transaction->add_activity($pay_method, $activity_name);
        $activity->do_activity();
        $transaction->update_status($activity->timestamp);
      }
      unset($form_state['rebuild'], $form_state['storage']);
    }
  }
}
