<?php // $Id: pay.form.inc,v 1.5 2009/08/21 02:49:22 vauxia Exp $

/**
 * @file
 * Payment form functions for the pay module.
 */

/**
 * The process function for a form element of type 'pay'.  This expands the
 * element into a proper payment form, based on on the configuration supplied
 * by the module using this element.
 */
function pay_form_element_process(&$element, $edit, &$form_state, $form) {
  // In order to function, we minimally need #pay_type.
  if (!$type = $element['#pay_type']) return FALSE;

  // We also need the handler, if it exists.
  if (!$handler = $element['#pay_handler']) $handler = $type;

  // pay_load accepts a unique identifier or just the handler's name.
  if (!$id = $element['#'. $type .'_id']) $id = $handler;

  // Load the pay handler that is being referenced in this element.
  if ($pay = pay_load($type, $id)) {
    $element = array_merge($element, $pay->form($form_state, $edit));
  }
  $element['#tree'] = TRUE;

  // Ensure that our form handlers are added to the form. This makes it so
  // that any form can be a fully-functional payment handling form.
  if (!is_array($form_state['submit_handllers'])
    || !in_array('pay_form_submit', $form_state['submit_handlers'])) {
    $form_state['validate_handlers'][] = 'pay_form_validate';
    $form_state['submit_handlers'][] = 'pay_form_submit';
  }

  return $element;
}

/**
 * Validate handler for any form that contains a pay element.
 */
function pay_form_validate($form, &$form_state) {
  foreach (element_children($form) as $key) {
    if ($type = $form[$key]['#pay_type']) {
      $pay = pay_load($type, $form[$key]['#pay_form_id']);
      $pay->form_validate($form, $form_state, $key);
    }
  }
}

/**
 * Submit handler for any form that contains a pay element.
 */
function pay_form_submit($form, &$form_state) {
  foreach (element_children($form) as $key) {
    if ($type = $form[$key]['#pay_type']) {
      $pay = pay_load($type, $form[$key]['#pay_form_id']);
      $pay->form_submit($form, $form_state, $key);
    }
  }
}
