<?php // $Id: pay_donate.inc,v 1.8 2009/08/21 02:49:22 vauxia Exp $

class pay_donate extends pay_form {

  var $menu_path;
  var $header_text;
  var $header_format = FILTER_FORMAT_DEFAULT;
  var $other_amount = TRUE;
  var $recommended_amounts;
  var $footer_text;
  var $footer_format = FILTER_FORMAT_DEFAULT;
  var $confirmation_text = 'Thank you for your contribution.';
  var $confirmation_format = FILTER_FORMAT_DEFAULT;
  var $confirmation_path = '';

  function recommended_amounts() {
    $amounts = array();
    if ($this->recommended_amounts) {
      foreach (explode("\n", $this->recommended_amounts) as $item) {
        list($key, $val) = explode('|', $item);
        if (!$val) $val = $key;
        $amounts[(float)trim(filter_xss($key))] = trim(filter_xss($val));
      }
    }
    return $amounts;
  }

  function settings_form(&$form_state) {
    $form = parent::settings_form($form_state);

    $form['donate'] = array(
      '#type' => 'fieldset',
      '#title' => t('Donation settings'),
      '#tree' => FALSE,
    );

    $form['donate']['menu_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Menu path'),
      '#default_value' => $this->menu_path,
    );

    $form['donate']['header']['header_text'] = array(
      '#type' => 'textarea',
      '#title' => t('Header text'),
      '#description' => t('Some text that will appear before the payment information'),
      '#default_value' => $this->header_text,
    );
    $form['donate']['header']['format'] = filter_form($this->header_format, NULL, array('header_format'));

    $form['donate']['recommended_amounts'] = array(
      '#type' => 'textarea',
      '#title' => t('Recommended amounts'),
      '#default_value' => $this->recommended_amounts,
    );

    $form['donate']['other_amount'] = array(
      '#type' => 'checkbox',
      '#title' => t('Allow visitors to choose an amount'),
      '#description' => t('Add a freeform "Other amount" field that contributors can populate.'),
      '#default_value' => $this->other_amount,
    );

    $form['donate']['total_goal'] = array(
      '#type' => 'textfield',
      '#size' => 10,
      '#title' => t('Total goal'),
      '#description' => t('The goal amount for this donation form.  This is optional and useful for tracking purposes.'),
      '#default_value' => $this->total_goal,
    );

    $form['donate']['footer']['footer_text'] = array(
      '#type' => 'textarea',
      '#title' => t('Footer text'),
      '#description' => t('Some text that will appear after the payment information.'),
      '#default_value' => $this->footer_text,
    );
    $form['donate']['footer']['format'] = filter_form($this->footer_format, NULL, array('footer_format'));

    $form['donate']['confirmation']['confirmation_text'] = array(
      '#type' => 'textarea',
      '#title' => t('Confirmation text'),
      '#description' => t('Assuming that everything went well, provide some "thank you" or other confirmation text.  If you enter a redirect URL, this message will appear as a standard drupal message'),
      '#default_value' => $this->confirmation_text,
    );

    $form['donate']['confirmation']['format'] = filter_form($this->confirmation_format, NULL, array('confirmation_format'));

    $form['donate']['confirmation_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Confirmation path'),
      '#default_value' => $this->confirmation_path,
      '#description' => t('Enter a drupal path or a fully-qualified URL.  If you omit this, the confirmation message, above, will be displayed.'),
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save form'),
    );
    return $form;
  }

  function settings_form_validate($form, &$form_state) {
    parent::settings_form_validate($form, $form_state);
  }

  function settings_form_submit($form, &$form_state) {
    parent::settings_form_submit($form, $form_state);

    // Do a menu rebuild to pick up any new menu paths.
    menu_rebuild();
  }

  function form(&$form_state, $values) {
    drupal_add_js(drupal_get_path('module','pay_donate') .'/js/pay_donate.js');
    $form = parent::form($form_state, $values);

    if ($this->header_text) {
      $form['header_text'] = array(
        '#type' => 'markup',
        '#value' => check_markup($this->header_text, $this->header_format),
      );
    }

    if ($amounts = $this->recommended_amounts()) {
      if ($this->other_amount) {
        $amounts['other'] = t('Other amount');
      }
      $form['amount'] = array(
        '#name' => 'amount',
        '#type' => 'radios',
        '#options' => $amounts,
        '#title' => t('Donation amount'),
      );

      if ($this->other_amount) {
        $form['amount_other'] = array(
          '#type' => 'textfield',
          '#title' => t('Other amount'),
          '#size' => 10,
          '#attributes' => array('class' => 'pay-donate-other-amount'),
        );
      }

      // Omit radio buttons if there's only 1 amount to select from.
      if (count($amounts) == 1) {
        $form['amount']['#access'] = FALSE;
        $form['amount']['#default_value'] = array_keys($amounts);
        $form['amount_display'] = array(
          '#type' => 'markup',
          '#value' => $label,
          '#attributes' => array('class' => 'pay-donate-amount'),
        );
      }
    }
    else {
      $form['amount_other'] = array(
        '#type' => 'textfield',
        '#title' => t('Donation amount'),
        '#size' => 10,
      );
    }

    // Ensure there is a "total" field, but populate it on the validate hook.
    $form['total'] = array(
      '#type' => 'textfield',
      '#access' => FALSE,
    );

    $form['pay_method'] = $this->pay_method_form($form_state);

    $form['notes'] = array(
      '#type' => 'textarea',
      '#title' => $this->notes_title,
      '#description' => $this->notes_description,
    );

    if ($this->footer_text) {
      $form['footer_text'] = array(
        '#type' => 'markup',
        '#value' => check_markup($this->footer_text, $this->footer_format),
      );
    }

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Submit contribution'),
    );

    if (isset($form_state['storage']['confirmation'])) {
      $form = array(
        '#type' => 'markup',
        '#value' => check_markup($this->confirmation_text, $this->confirmation_format),
      );
    }

    return $form;
  }

  function form_validate($form, &$form_state, $key) {

    // Use the "other amount" value if it's supplied.
    $values = $form_state['values'][$key];
    if (!empty($values['amount_other'])) {
      $total = $values['amount_other'];
    }

    // Alternatively, use the amount selected value from radio buttons.
    elseif($values['amount']) {
      $total = $values['amount'];
    }

    // Set the form value so our parent thinks it was there all along.
    form_set_value($form[$key]['total'], (float)$total, $form_state);

    // Call the parent validator to handle amounts and other inputs.
    parent::form_validate($form, $form_state, $key);
  }

  function form_submit($form, &$form_state, $key) {
    parent::form_submit($form, $form_state, $key);

    if (!isset($form_state['rebuild'])) {
      if (!isset($form_state['redirect'])) {
        if ($this->confirmation_path) {
          drupal_set_message(check_markup($this->confirmation_path, $this->confirmation_format));
          $form_state['redirect'] = $this->confirmation_path;
        }
        else {
          $form_state['storage']['confirmation'] = TRUE;
          $form_state['rebuild'] = TRUE;
        }
      }
    }
  }
}
